use super::RepoSharedState;
use crate::db;

use std::{
    collections::HashMap,
    path::{Path, PathBuf},
    sync::{atomic::AtomicU32, Arc, Mutex, RwLock},
};

use sea_orm::{
    ActiveModelTrait, ColumnTrait, Condition, ConnectionTrait, DbConn, EntityTrait, JoinType,
    ModelTrait, NotSet, QueryFilter, QuerySelect, Related, RelationTrait, Set, TransactionTrait,
};
use tokio::{
    runtime,
    sync::mpsc::{unbounded_channel, UnboundedSender},
};

#[derive(Clone)]
pub struct Handle {
    state: Arc<RepoSharedState>,
}

impl Handle {
    pub fn start(
        repos_dir: impl AsRef<Path>,
        conn: DbConn,
        rt: runtime::Handle,
        actors: u32,
    ) -> crate::Result<Self> {
        std::fs::create_dir_all(repos_dir.as_ref())?;

        let mut repos = HashMap::new();
        let repo_ids: Vec<i32> = rt.block_on(
            db::Repo::find()
                .select_only()
                .column(db::repo::Column::Id)
                .into_tuple()
                .all(&conn),
        )?;

        for id in repo_ids {
            repos.insert(id, Default::default());
        }

        let state = Arc::new(RepoSharedState::new(repos_dir, conn, repos));

        for _ in 0..actors {
            let actor = super::RepoActor::new(rt.clone(), Arc::clone(&state));

            std::thread::spawn(|| actor.run());
        }

        Ok(Self { state })
    }
}
